package live.exit.musicplayer

import android.R.drawable.ic_media_previous
import android.R.drawable.ic_media_pause
import android.R.drawable.ic_media_play
import android.R.drawable.ic_media_next

import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import android.graphics.Bitmap
import android.support.v4.app.NotificationCompat
import android.support.v4.media.session.MediaSessionCompat

class NotificationHandler(private val context: Context, private val session: MediaSessionCompat) {
  private var notificationManager: NotificationManager? = null
  private var isPlaying = true
  private var coverImage: Bitmap? = null
  private var notificationIcon: Int? = ic_media_play
  private var trackName: String? = ""
  private var albumName: String? = ""
  private var artistName: String? = ""

  private val pause = Intent(NOTIFY_PAUSE)
  private val pPause = PendingIntent.getBroadcast(context, 1, pause, PendingIntent.FLAG_UPDATE_CURRENT)
  private val play = Intent(NOTIFY_PLAY)
  private val pPlay = PendingIntent.getBroadcast(context, 1, play, PendingIntent.FLAG_UPDATE_CURRENT)
  private val previous = Intent(NOTIFY_PREVIOUS)
  private val pPrevious = PendingIntent.getBroadcast(context, 1, previous, PendingIntent.FLAG_UPDATE_CURRENT)
  private val next = Intent(NOTIFY_NEXT)
  private val pNext = PendingIntent.getBroadcast(context, 1, next, PendingIntent.FLAG_UPDATE_CURRENT)

  init {
    notificationManager = context.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
  }

  fun updateNotification(isPlaying: Boolean) {
    this.isPlaying = isPlaying
    updateNotification()
  }

  fun updateNotification(image: Bitmap) {
    this.coverImage = image
    updateNotification()
  }

  fun updateNotification(trackName: String, albumName: String, artistName: String) {
    this.trackName = trackName
    this.albumName = albumName
    this.artistName = artistName
    updateNotification()
  }

  private fun updateNotification() {
    if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
      val id = "music_player_notifications"
      val name = "Music Player Notification Channel"
      val mChannel = NotificationChannel(id, name, NotificationManager.IMPORTANCE_LOW)
      notificationManager?.createNotificationChannel(mChannel)

      val builder = NotificationCompat.Builder(context, id)
          .setSmallIcon(notificationIcon!!)
          .setLargeIcon(coverImage)
          .setSubText(albumName)
          .setContentTitle(trackName)
          .setContentText(artistName)

      builder.addAction(ic_media_previous, "Previous", pPrevious)
      if (isPlaying) {
        builder.addAction(ic_media_pause, "Pause", pPause)
      } else {
        builder.addAction(ic_media_play, "Play", pPlay)
      }
      builder.addAction(ic_media_next, "Next", pNext)

      builder.setStyle(android.support.v4.media.app.NotificationCompat.MediaStyle().setMediaSession(session.sessionToken))
      notificationManager?.notify(1, builder.build())
    }
  }
}

